#pragma NOIV                    // Do not generate interrupt vectors
//-----------------------------------------------------------------------------
//   File:      slave.c
//   Contents:  Hooks required to implement USB peripheral function.
//              Code written for FX2 REVE 56-pin and above.
//              This firmware is used to demonstrate FX2 Slave FIF
//              operation.
//   Copyright (c) 2003 Cypress Semiconductor All rights reserved
//-----------------------------------------------------------------------------
#include "fx2.h"
#include "fx2regs.h"
#include "fx2sdly.h"            // SYNCDELAY macro

#define LED_ALL         (bmBIT0 | bmBIT1 | bmBIT2 | bmBIT3)

extern BOOL GotSUD;             // Received setup data flag
extern BOOL Sleep;
extern BOOL Rwuen;
extern BOOL Selfpwr;

BYTE Configuration;             // Current configuration
BYTE AlternateSetting;          // Alternate settings
static WORD xdata LED_Count = 0;
static BYTE xdata LED_Status = 0;




//-----------------------------------------------------------------------------
// Task Dispatcher hooks
//   The following hooks are called by the task dispatcher.
//-----------------------------------------------------------------------------
void TD_Init( void )
{ // Called once at startup
   // set the CPU clock to 48MHz
  // CPUCS = ((CPUCS & ~bmCLKSPD) | bmCLKSPD1) ;

   // set the CPU clock to 24MHz which is what the FPGA is putting out
   CPUCS = ((CPUCS & ~bmCLKSPD) | bmCLKSPD0) ;
   SYNCDELAY;
  FIFOPINPOLAR = 0x00; // PKTEND:SLOE:SLRD:SLWR:EF:FF =0  all lines Active low, 
	SYNCDELAY;
  PINFLAGSAB = 0x00;// FLAGA and FLAGB are indexed
  SYNCDELAY;
  
  PINFLAGSCD = 0x00; // FLAGC and FLAGD are indexed (however, FLAGD is not used, and CS is used 
  SYNCDELAY;
  
  PORTACFG = 0x40; // B6 is set to 1 for SLCS_bar, FLAGD disabled and no interrupt
  SYNCDELAY;
  

   IFCONFIG = 0x43; // IFCFG[1:0] st to 11 for Slave FIFO interface (external master 
	SYNCDELAY;
  // IFCLKSRC=0   , FIFOs executes on external clk source 
  // xMHz=1       , 48MHz operation
  // IFCLKOE=0    ,Drive IFCLK pin signal at 48MHz
  // IFCLKPOL=0   ,  do not invert IFCLK pin signal from internal clk
  // ASYNC=0      , master samples synchronous
  // GSTATE=0     , Don't drive GPIF states out on PORTE[2:0], debug WF
  // IFCFG[1:0]=11, FX2 in slave FIFO mode

  
  REVCTL = 0x03; // REVCTL.0 and REVCTL.1 to set to 1 See section 9.3.4 Auto arming is disabled. 
  //The core disables auto-arming of the endpoints when AUTOOUT transitions from 0 to 1.
//This feature allows CPU intervention when switching between AUTO and Manual mode
//without having to reset the endpoint.
//Note When DYN_OUT = 1 and AUTOOUT = 1, the CPU is responsible for priming the
//pump by initially arming the endpoints (OUTPKTEND w/SKIP = 1 to pass packets to host).

/*
The CPU has additional capabilities:
OUT packets: Skip, Commit, or Edit/Source
IN packets: Skip, Commit, or Edit/Source

*/
  SYNCDELAY;





  // Registers which require a synchronization delay, see section 15.14
  // FIFORESET        FIFOPINPOLAR
  // INPKTEND         OUTPKTEND
  // EPxBCH:L         REVCTL
  // GPIFTCB3         GPIFTCB2
  // GPIFTCB1         GPIFTCB0
  // EPxFIFOPFH:L     EPxAUTOINLENH:L
  // EPxFIFOCFG       EPxGPIFFLGSEL
  // PINFLAGSxx       EPxFIFOIRQ
  // EPxFIFOIE        GPIFIRQ
  // GPIFIE           GPIFADRH:L
  // UDMACRCH:L       EPxGPIFTRIG
  // GPIFTRIG
  
  // Note: The pre-REVE EPxGPIFTCH/L register are affected, as well...
  //      ...these have been replaced by GPIFTC[B3:B0] registers

  // default: all endpoints have their VALID bit set
  // default: TYPE1 = 1 and TYPE0 = 0 --> BULK  
  // default: EP4 and EP8 DIR bits are 0 (OUT direction)
  // default: EP2 and EP6 DIR bits are 1 (IN direction)
  // default: EP2, EP4, EP6, and EP8 are double buffered
  
   // we are just using the default values, yes this is not necessary...
  EP1OUTCFG = 0xA0;
  SYNCDELAY;
  EP1INCFG = 0xA0;
  SYNCDELAY;                    // see TRM section 15.14

	
	// set wakeupcs to change the polarity of wakeup pin
	// in our design its always pulled high to 3.3 changing the polatity to active high sets the cypress to never go into suspend state
	WAKEUPCS |= 0x09; // WUPOL is set to active high, WUEN is set to 1
	SYNCDELAY;
                   
  EP2CFG        = 0xE2;    //valid = activate 1,  DIR =IN 1, TYPE[1:0]=10 bulk,SIZE = 0 512bytes,0,BUF[1:0] 10 double buffering;  1 1 10 0 0 10 = 0xE2
  SYNCDELAY;                    
  EP4CFG = 0xA2; // valid = 1 activate, DIR = OUT 0, TYPE[1:0] = 10 bulk, SIZE = 0 512bytes,0,BUF[1:0] 10 double buffering;  1 0 10 0 0 10 = 0xA2
  SYNCDELAY;                    
  EP6CFG = 0xE2; //valid = activate 1,  DIR =IN 1, TYPE[1:0]=10 bulk,SIZE = 0 512bytes,0,BUF[1:0] 10 double buffering;  1 1 10 0 0 10 = 0xE2
  SYNCDELAY;                    
  EP8CFG = 0xA2; // valid = 1 activate, DIR = OUT 0, TYPE[1:0] = 10 bulk, SIZE = 0 512bytes,0,BUF[1:0] 10 double buffering;  1 0 10 0 0 10 = 0xA2
  SYNCDELAY;
   
  FIFORESET = 0x80;             // activate NAK-ALL[7] to avoid race conditions
  SYNCDELAY;                    // see TRM section 15.14
  FIFORESET = 0x02;             // reset, FIFO 2
  SYNCDELAY;                    // 
  FIFORESET = 0x04;             // reset, FIFO 4
  SYNCDELAY;                    // 
  FIFORESET = 0x06;             // reset, FIFO 6
  SYNCDELAY;                    // 
  FIFORESET = 0x08;             // reset, FIFO 8
  SYNCDELAY;                    // 
  FIFORESET = 0x00;             // deactivate NAK-ALL

  INPKTEND = 0x82; // Arm both EP2 buffers to prime the pump
  SYNCDELAY;
  INPKTEND = 0x82;
  SYNCDELAY;	

  OUTPKTEND = 0x84; // Arm both EP4 buffers to prime the pump
  SYNCDELAY;
  OUTPKTEND = 0x84;
  SYNCDELAY;

  INPKTEND = 0x86; // Arm both EP6 buffers to prime the pump
  SYNCDELAY;
  INPKTEND = 0x86;
  SYNCDELAY;

  OUTPKTEND = 0x88; // Arm both EP8 buffers to prime the pump
  SYNCDELAY;
  OUTPKTEND = 0x88;
  SYNCDELAY;


  
  // old code
/*
  // core needs to see AUTOOUT=0 to AUTOOUT=1 switch to arm endp's  
  EP2FIFOCFG = 0x00;// 0,INFM1[6] =0 InFull minus one disable, OEP1[5] = 0 out empty plus one disable, AUTOOUT[4] =0 , AUTOIN[3]=0, ZEROLENIN=0 zero length packets are sent on PKTEND,0, WORDWIDE =0 (byte size data) :0000 0000 
  SYNCDELAY;                    // 

  EP4FIFOCFG = 0x20;// 0,INFM1[6] =0 InFull minus one disable, OEP1[5] = 1 out empty plus one disable, AUTOOUT[4] =0 , AUTOIN[3]=0, ZEROLENIN=0 zero length packets are sent on PKTEND,0, WORDWIDE =0 (byte size data) :0010 0000 
  SYNCDELAY;                    // 

  EP6FIFOCFG = 0x00;// 0,INFM1[6] =0 InFull minus one disable, OEP1[5] = 0 out empty plus one disable, AUTOOUT[4] =0 , AUTOIN[3]=0, ZEROLENIN=0 zero length packets are sent on PKTEND,0, WORDWIDE =0 (byte size data) :0000 0000 
  SYNCDELAY;                    // 

  EP8FIFOCFG = 0x20;// 0,INFM1[6] =0 InFull minus one disable, OEP1[5] = 1 out empty plus one disable, AUTOOUT[4] =0 , AUTOIN[3]=0, ZEROLENIN=0 zero length packets are sent on PKTEND,0, WORDWIDE =0 (byte size data) :0010 0000 
  SYNCDELAY;
  
  AUTOPTRSETUP |= 0x01;// enables the auto pointer by setting APTREN bit to 1
  SYNCDELAY;
*/


    // core needs to see AUTOOUT=0 to AUTOOUT=1 switch to arm endp's  
  EP2FIFOCFG = 0x0C;// 0,INFM1[6] =0 InFull minus one disable, OEP1[5] = 0 out empty plus one disable, AUTOOUT[4] =0 , AUTOIN[3]=1, ZEROLENIN=1 zero length packets are sent on PKTEND,0, WORDWIDE =0 (byte size data) :0000 1100 
  SYNCDELAY;                    // 

  EP4FIFOCFG = 0x30;// 0,INFM1[6] =0 InFull minus one disable, OEP1[5] = 1 out empty plus one enable, AUTOOUT[4] =1 , AUTOIN[3]=0, ZEROLENIN=0 zero length packets are sent on PKTEND,0, WORDWIDE =0 (byte size data) :0011 0000 
  SYNCDELAY;                    // 

  EP6FIFOCFG = 0x0C;// 0,INFM1[6] =0 InFull minus one disable, OEP1[5] = 0 out empty plus one disable, AUTOOUT[4] =0 , AUTOIN[3]=1, ZEROLENIN=1 zero length packets are sent on PKTEND,0, WORDWIDE =0 (byte size data) :0000 1100 
  SYNCDELAY;                    // 

  EP8FIFOCFG = 0x30;// 0,INFM1[6] =0 InFull minus one disable, OEP1[5] = 1 out empty plus one enable, AUTOOUT[4] =1 , AUTOIN[3]=0, ZEROLENIN=0 zero length packets are sent on PKTEND,0, WORDWIDE =0 (byte size data) :0011 0000 
  SYNCDELAY;
  



  AUTOPTRSETUP |= 0x01;// enables the auto pointer by setting APTREN bit to 1
  SYNCDELAY;


	
}

void TD_Poll( void )
{ // Called repeatedly while the device is idle


// check all endpoint buffers for data and commit them continuously since we are not in autoin or autoout mode
// use OUTPKTEND 
/*
	// EP4
	if( !( EP2468STAT & bmEP4EMPTY ) )
	{ // EP4EF=0 when FIFO NOT empty, host sent packet
	OUTPKTEND = 0x02; // SKIP=0, pass buffer on to master
	SYNCDELAY;
	}
	// EP8
	if( !( EP2468STAT & bmEP8EMPTY ) )
	{ // EP8EF=0 when FIFO NOT empty, host sent packet
	OUTPKTEND = 0x08; // SKIP=0, pass buffer on to master
	SYNCDELAY;
	}

		// EP2
	if( !( EP2468STAT & bmEP2FULL ) )
	{ // EP2EF=0 when FIFO NOT empty, host sent packet
	OUTPKTEND = 0x01; // SKIP=0, pass buffer on to master
	SYNCDELAY;
	}
	// EP6
	if( !( EP2468STAT & bmEP6FULL ) )
	{ // EP6FF=0 when FIFO NOT empty, host sent packet
	OUTPKTEND = 0x04; // SKIP=0, pass buffer on to master
	SYNCDELAY;
	}
*/
	if (PC0 == 0) 
	{
		PC0 = 1; 
	} else {
		PC0 = 0;
	}

}

BOOL TD_Suspend( void )          
{ // Called before the device goes into suspend mode
	if (PC1 == 0) 
	{
		PC1 = 1; 
	} else {
		PC1 = 0;
	}
	/*
  FIFORESET = 0x80;             // activate NAK-ALL[7] to avoid race conditions
  SYNCDELAY;                    // see TRM section 15.14
  FIFORESET = 0x02;             // reset, FIFO 2
  SYNCDELAY;                    // 
  FIFORESET = 0x04;             // reset, FIFO 4
  SYNCDELAY;                    // 
  FIFORESET = 0x06;             // reset, FIFO 6
  SYNCDELAY;                    // 
  FIFORESET = 0x08;             // reset, FIFO 8
  SYNCDELAY;                    // 
  FIFORESET = 0x00;             // deactivate NAK-ALL
*/


   return( TRUE );
}

BOOL TD_Resume( void )          
{ // Called after the device resumes
	if (PC2 == 0) 
	{
		PC2 = 1; 
	} else {
		PC2 = 0;
	}

/*
  INPKTEND = 0x82; // Arm both EP2 buffers to prime the pump
  SYNCDELAY;
  INPKTEND = 0x82;
  SYNCDELAY;	

  OUTPKTEND = 0x84; // Arm both EP4 buffers to prime the pump
  SYNCDELAY;
  OUTPKTEND = 0x84;
  SYNCDELAY;

  INPKTEND = 0x86; // Arm both EP6 buffers to prime the pump
  SYNCDELAY;
  INPKTEND = 0x86;
  SYNCDELAY;

  OUTPKTEND = 0x88; // Arm both EP8 buffers to prime the pump
  SYNCDELAY;
  OUTPKTEND = 0x88;
  SYNCDELAY;
*/
 
	return( TRUE );
}

//-----------------------------------------------------------------------------
// Device Request hooks
//   The following hooks are called by the end point 0 device request parser.
//-----------------------------------------------------------------------------
BOOL DR_GetDescriptor( void )
{
   return( TRUE );
}

BOOL DR_SetConfiguration( void )   
{ // Called when a Set Configuration command is received 

  if( EZUSB_HIGHSPEED( ) )
  { // ...FX2 in high speed mode
    EP4AUTOINLENH = 0x02;
    SYNCDELAY;
    EP8AUTOINLENH = 0x02;   // set core AUTO commit len = 512 bytes
    SYNCDELAY;
    EP4AUTOINLENL = 0x00;
    SYNCDELAY;
    EP8AUTOINLENL = 0x00;
  }
  else
  { // ...FX2 in full speed mode
    EP4AUTOINLENH = 0x00;
    SYNCDELAY;
    EP8AUTOINLENH = 0x00;   // set core AUTO commit len = 64 bytes
    SYNCDELAY;
    EP4AUTOINLENL = 0x40;
    SYNCDELAY;
    EP8AUTOINLENL = 0x40;
  }


  Configuration = SETUPDAT[ 2 ];
  return( TRUE );        // Handled by user code
}

BOOL DR_GetConfiguration( void )   
{ // Called when a Get Configuration command is received
   EP0BUF[ 0 ] = Configuration;
   EP0BCH = 0;
   EP0BCL = 1;
   return(TRUE);          // Handled by user code
}

BOOL DR_SetInterface( void )       
{ // Called when a Set Interface command is received
   AlternateSetting = SETUPDAT[ 2 ];
   return( TRUE );        // Handled by user code
}

BOOL DR_GetInterface( void )       
{ // Called when a Set Interface command is received
   EP0BUF[ 0 ] = AlternateSetting;
   EP0BCH = 0;
   EP0BCL = 1;
   return( TRUE );        // Handled by user code
}

BOOL DR_GetStatus( void )
{
   return( TRUE );
}

BOOL DR_ClearFeature( void )
{
   return( TRUE );
}

BOOL DR_SetFeature( void )
{
   return( TRUE );
}

BOOL DR_VendorCmnd( void )
{
  return( TRUE );
}

//-----------------------------------------------------------------------------
// USB Interrupt Handlers
//   The following functions are called by the USB interrupt jump table.
//-----------------------------------------------------------------------------

// Setup Data Available Interrupt Handler
void ISR_Sudav( void ) interrupt 0
{
   GotSUD = TRUE;         // Set flag
   EZUSB_IRQ_CLEAR( );
   USBIRQ = bmSUDAV;      // Clear SUDAV IRQ
}

// Setup Token Interrupt Handler
void ISR_Sutok( void ) interrupt 0
{
   EZUSB_IRQ_CLEAR( );
   USBIRQ = bmSUTOK;      // Clear SUTOK IRQ
}

void ISR_Sof( void ) interrupt 0
{
   EZUSB_IRQ_CLEAR( );
   USBIRQ = bmSOF;        // Clear SOF IRQ
}

void ISR_Ures( void ) interrupt 0
{
   if ( EZUSB_HIGHSPEED( ) )
   {
      pConfigDscr = pHighSpeedConfigDscr;
      pOtherConfigDscr = pFullSpeedConfigDscr;
   }
   else
   {
   pConfigDscr = pFullSpeedConfigDscr;
   pOtherConfigDscr = pHighSpeedConfigDscr;
   }
   
   EZUSB_IRQ_CLEAR( );
   USBIRQ = bmURES;       // Clear URES IRQ

}

void ISR_Susp( void ) interrupt 0
{
  // Sleep = TRUE; // VS debug deliberately preventing the cypress from going into suspend mode when the usb is disconnected
   EZUSB_IRQ_CLEAR( );
   USBIRQ = bmSUSP;
}

void ISR_Highspeed( void ) interrupt 0
{
   if (EZUSB_HIGHSPEED())
   {
      pConfigDscr = pHighSpeedConfigDscr;
      pOtherConfigDscr = pFullSpeedConfigDscr;
   }
   else
   {
      pConfigDscr = pFullSpeedConfigDscr;
      pOtherConfigDscr = pHighSpeedConfigDscr;
   }

   EZUSB_IRQ_CLEAR( );
   USBIRQ = bmHSGRANT;
}
void ISR_Ep0ack( void ) interrupt 0
{
}
void ISR_Stub( void ) interrupt 0
{
}
void ISR_Ep0in( void ) interrupt 0
{
}
void ISR_Ep0out( void ) interrupt 0
{
}
void ISR_Ep1in( void ) interrupt 0
{
}
void ISR_Ep1out( void ) interrupt 0
{
}
void ISR_Ep2inout( void ) interrupt 0
{
}
void ISR_Ep4inout( void ) interrupt 0
{
}
void ISR_Ep6inout( void ) interrupt 0
{
}
void ISR_Ep8inout( void ) interrupt 0
{
}
void ISR_Ibn( void ) interrupt 0
{
}
void ISR_Ep0pingnak( void ) interrupt 0
{
}
void ISR_Ep1pingnak( void ) interrupt 0
{
}
void ISR_Ep2pingnak( void ) interrupt 0
{
}
void ISR_Ep4pingnak( void ) interrupt 0
{
}
void ISR_Ep6pingnak( void ) interrupt 0
{
}
void ISR_Ep8pingnak( void ) interrupt 0
{
}
void ISR_Errorlimit( void ) interrupt 0
{
}
void ISR_Ep2piderror( void ) interrupt 0
{
}
void ISR_Ep4piderror( void ) interrupt 0
{
}
void ISR_Ep6piderror( void ) interrupt 0
{
}
void ISR_Ep8piderror( void ) interrupt 0
{
}
void ISR_Ep2pflag( void ) interrupt 0
{
}
void ISR_Ep4pflag( void ) interrupt 0
{
}
void ISR_Ep6pflag( void ) interrupt 0
{
}
void ISR_Ep8pflag( void ) interrupt 0
{
}
void ISR_Ep2eflag( void ) interrupt 0
{
}
void ISR_Ep4eflag( void ) interrupt 0
{
}
void ISR_Ep6eflag( void ) interrupt 0
{
}
void ISR_Ep8eflag( void ) interrupt 0
{
}
void ISR_Ep2fflag( void ) interrupt 0
{
}
void ISR_Ep4fflag( void ) interrupt 0
{
}
void ISR_Ep6fflag( void ) interrupt 0
{
}
void ISR_Ep8fflag( void ) interrupt 0
{
}
void ISR_GpifComplete( void ) interrupt 0
{
}
void ISR_GpifWaveform( void ) interrupt 0
{
}



